"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var compiler_1 = require("@angular/compiler");
var env_1 = require("./env");
var Visitor = /** @class */ (function () {
    function Visitor() {
    }
    Visitor.prototype.unparse = function (ast) {
        this.properties = [];
        this.graph = [];
        this._visit(ast);
        return this.properties;
    };
    Visitor.prototype.visitPropertyRead = function (ast, context) {
        this.graph.push(ast.name);
        this._visit(ast.receiver);
    };
    Visitor.prototype.visitPropertyWrite = function (ast, context) {
        this._visit(ast.receiver);
        this._visit(ast.value);
    };
    Visitor.prototype.visitBinary = function (ast, context) {
        this._visit(ast.left);
        this._visit(ast.right);
    };
    Visitor.prototype.visitChain = function (ast, context) {
        var len = ast.expressions.length;
        for (var i = 0; i < len; i++) {
            this._visit(ast.expressions[i]);
        }
    };
    Visitor.prototype.visitConditional = function (ast, context) {
        this._visit(ast.condition);
        this._visit(ast.trueExp);
        this._visit(ast.falseExp);
    };
    Visitor.prototype.visitPipe = function (ast, context) {
        var _this = this;
        this._visit(ast.exp);
        ast.args.forEach(function (arg) {
            _this._visit(arg);
        });
    };
    Visitor.prototype.visitFunctionCall = function (ast, context) {
        var _this = this;
        this._visit(ast.target);
        ast.args.forEach(function (arg) {
            _this._visit(arg);
        });
    };
    Visitor.prototype.visitImplicitReceiver = function (ast, context) {
        var _this = this;
        this.properties.push(this.graph.reduceRight(function (expression, token, index) {
            switch (token) {
                case '[':
                    return expression + "]";
                case ']':
                    return expression + "[";
                default:
                    return expression
                        ? _this.graph[index - 1] == '['
                            ? "" + expression + token
                            : expression + "." + token
                        : token;
            }
        }, ''));
        this.graph = [];
    };
    Visitor.prototype.visitInterpolation = function (ast, context) {
        for (var i = 0; i < ast.strings.length; i++) {
            if (i < ast.expressions.length) {
                this._visit(ast.expressions[i]);
            }
        }
    };
    Visitor.prototype.visitKeyedRead = function (ast, context) {
        this.graph.push('[');
        this._visit(ast.key);
        this.graph.push(']');
        this._visit(ast.obj);
    };
    Visitor.prototype.visitKeyedWrite = function (ast, context) {
        this._visit(ast.obj);
        this._visit(ast.key);
        this._visit(ast.value);
    };
    Visitor.prototype.visitLiteralArray = function (ast, context) {
        var _this = this;
        ast.expressions.forEach(function (expression) {
            _this._visit(expression);
        });
    };
    Visitor.prototype.visitLiteralMap = function (ast, context) {
        for (var i = 0; i < ast.keys.length; i++) {
            this._visit(ast.values[i]);
        }
    };
    Visitor.prototype.visitLiteralPrimitive = function (ast, context) {
        if (this.graph[this.graph.length - 1] == '[') {
            this.graph.push(ast.value);
        }
    };
    Visitor.prototype.visitMethodCall = function (ast, context) {
        var _this = this;
        this.graph.push(ast.name);
        this._visit(ast.receiver);
        ast.args.forEach(function (arg) {
            _this._visit(arg);
        });
    };
    Visitor.prototype.visitPrefixNot = function (ast, context) {
        this._visit(ast.expression);
    };
    Visitor.prototype.visitNonNullAssert = function (ast, context) {
        this._visit(ast.expression);
    };
    Visitor.prototype.visitSafePropertyRead = function (ast, context) {
        this.graph.push(ast.name);
        this._visit(ast.receiver);
    };
    Visitor.prototype.visitSafeMethodCall = function (ast, context) {
        var _this = this;
        this._visit(ast.receiver);
        ast.args.forEach(function (arg) {
            _this._visit(arg);
        });
    };
    Visitor.prototype.visitQuote = function (ast, context) { };
    Visitor.prototype._visit = function (ast) {
        ast.visit(this);
    };
    return Visitor;
}());
function default_1(value) {
    var ast = new compiler_1.Parser(new compiler_1.Lexer()).parseInterpolation(value, null);
    if (ast != null) {
        var visitor = new Visitor();
        return visitor.unparse(ast);
    }
    return [];
}
exports.default = default_1;
function parseExpression(value) {
    return new compiler_1.Parser(new compiler_1.Lexer()).parseInterpolation(env_1.interpolateAsAttribute(null, value), null);
}
exports.parseExpression = parseExpression;
function isSimpleExpression(ast) {
    if (ast == null) {
        return false;
    }
    if (ast instanceof compiler_1.ASTWithSource) {
        ast = ast.ast;
    }
    if (ast instanceof compiler_1.Interpolation) {
        if (ast.expressions.length != 1) {
            return false;
        }
        var exp = ast.expressions[0];
        return isSimpleExpression(exp);
    }
    if (ast instanceof compiler_1.BindingPipe) {
        for (var _i = 0, _a = ast.args; _i < _a.length; _i++) {
            var arg = _a[_i];
            if (!(arg instanceof compiler_1.LiteralPrimitive)) {
                return false;
            }
        }
        return isSimpleExpression(ast.exp);
    }
    if (ast instanceof compiler_1.PropertyRead) {
        return true;
    }
    if (ast instanceof compiler_1.SafePropertyRead) {
        if (ast.receiver instanceof compiler_1.KeyedRead) {
            return false;
        }
        if (ast.receiver instanceof compiler_1.SafeMethodCall) {
            return false;
        }
        return ast.name != 'length';
    }
    return false;
}
exports.isSimpleExpression = isSimpleExpression;
var SimpleExpressionVisitor = /** @class */ (function () {
    function SimpleExpressionVisitor() {
        this.path = [];
    }
    SimpleExpressionVisitor.prototype.visitBinary = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitChain = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitConditional = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitFunctionCall = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitImplicitReceiver = function (ast, context) { };
    SimpleExpressionVisitor.prototype.visitInterpolation = function (ast, context) {
        for (var i = 0; i < ast.strings.length; i++) {
            if (i < ast.expressions.length) {
                ast.expressions[i].visit(this);
            }
        }
    };
    SimpleExpressionVisitor.prototype.visitKeyedRead = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitKeyedWrite = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitLiteralArray = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitLiteralMap = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitLiteralPrimitive = function (ast, context) {
        this.pipe.args.push(ast.value);
    };
    SimpleExpressionVisitor.prototype.visitMethodCall = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitPipe = function (ast, context) {
        var _this = this;
        ast.exp.visit(this);
        this.pipe = {
            name: ast.name,
            args: []
        };
        ast.args.forEach(function (arg) { return arg.visit(_this); });
    };
    SimpleExpressionVisitor.prototype.visitPrefixNot = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitNonNullAssert = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitPropertyRead = function (ast, context) {
        this.path.push(ast.name);
        ast.receiver.visit(this);
    };
    SimpleExpressionVisitor.prototype.visitPropertyWrite = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitQuote = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitSafeMethodCall = function (ast, context) {
        throw new Error('Method not implemented.');
    };
    SimpleExpressionVisitor.prototype.visitSafePropertyRead = function (ast, context) {
        this.path.push(ast.name);
        ast.receiver.visit(this);
    };
    SimpleExpressionVisitor.prototype.unparse = function (ast) {
        var result = {};
        ast.visit(this);
        result.path = this.path.reduceRight(function (result, token) { return (result ? result + "." + token : token); }, '');
        if (this.pipe) {
            result.pipe = this.pipe;
        }
        return result;
    };
    return SimpleExpressionVisitor;
}());
function parseSimpleExpression(ast) {
    return new SimpleExpressionVisitor().unparse(ast);
}
exports.parseSimpleExpression = parseSimpleExpression;
//# sourceMappingURL=property-parser.js.map