"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var operators_1 = require("rxjs/operators");
var core_1 = require("@angular/core");
var common_1 = require("@angular/common");
var animations_1 = require("@angular/animations");
var rxjs_1 = require("rxjs");
var transition_1 = require("./transition");
var layout_1 = require("./layout");
var SidebarComponent = /** @class */ (function () {
    function SidebarComponent(layoutService) {
        var _this = this;
        this.layoutService = layoutService;
        this.style = {};
        this.ngUnsubscribe = new rxjs_1.Subject();
        layoutService.headerHeight.pipe(operators_1.takeUntil(this.ngUnsubscribe))
            .subscribe(function (headerHeight) {
            _this.headerHeight = headerHeight;
        });
        layoutService.footerHeight.pipe(operators_1.takeUntil(this.ngUnsubscribe))
            .subscribe(function (footerHeight) {
            _this.footerHeight = footerHeight;
        });
        this.open = !this.layoutService.deviceMediaQuery().matches;
    }
    Object.defineProperty(SidebarComponent.prototype, "open", {
        get: function () {
            return this.layoutService.sidebarOpen.getValue();
        },
        set: function (value) {
            this.layoutService.sidebarOpen.next(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SidebarComponent.prototype, "stretch", {
        get: function () {
            return this.layoutService.sidebarStretch.getValue();
        },
        set: function (value) {
            this.layoutService.sidebarStretch.next(value);
        },
        enumerable: true,
        configurable: true
    });
    SidebarComponent.prototype.ngOnInit = function () {
        var nativeElement = this.sidebar.nativeElement;
        this.layoutService.sidebarWidth.next(nativeElement.offsetWidth);
    };
    SidebarComponent.prototype.ngOnChanges = function (changes) {
        var style = changes.style, open = changes.open, stretch = changes.stretch;
        if (open && !open.firstChange) {
            this.layoutService.sidebarOpen.next(open.currentValue);
        }
        if (stretch && !stretch.firstChange) {
            this.layoutService.sidebarStretch.next(stretch.currentValue);
        }
    };
    SidebarComponent.prototype.ngAfterViewInit = function () {
        this.layoutService.sidebarWidth.next(this.sidebar.nativeElement.offsetWidth);
    };
    SidebarComponent.prototype.ngOnDestroy = function () {
        this.open = false;
        this.ngUnsubscribe.next();
        this.ngUnsubscribe.complete();
    };
    __decorate([
        core_1.Input(),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [Boolean])
    ], SidebarComponent.prototype, "open", null);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [Boolean])
    ], SidebarComponent.prototype, "stretch", null);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], SidebarComponent.prototype, "style", void 0);
    __decorate([
        core_1.ViewChild('sidebar'),
        __metadata("design:type", core_1.ElementRef)
    ], SidebarComponent.prototype, "sidebar", void 0);
    SidebarComponent = __decorate([
        core_1.Component({
            selector: 'rz-sidebar',
            template: "\n    <div\n      #sidebar\n      [ngClass]=\"{'rz-sidebar': true}\"\n      [ngStyle]=\"style\"\n      [@open]=\"open ? 'visible' : 'hidden'\"\n      [style.top.px]=\"stretch ? 0 : headerHeight\"\n      [style.bottom.px]=\"stretch ? 0 : footerHeight\"\n    >\n      <ng-content></ng-content>\n    </div>\n  ",
            animations: [
                animations_1.trigger('open', [
                    animations_1.state('visible', animations_1.style({
                        transform: 'translateX(0)'
                    })),
                    animations_1.state('hidden', animations_1.style({
                        transform: 'translateX(-100%)'
                    })),
                    animations_1.transition('hidden <=> visible', animations_1.animate(transition_1.EASING))
                ])
            ]
        }),
        __metadata("design:paramtypes", [layout_1.LayoutService])
    ], SidebarComponent);
    return SidebarComponent;
}());
exports.SidebarComponent = SidebarComponent;
var SidebarModule = /** @class */ (function () {
    function SidebarModule() {
    }
    SidebarModule = __decorate([
        core_1.NgModule({
            imports: [common_1.CommonModule, layout_1.LayoutModule],
            declarations: [SidebarComponent],
            exports: [SidebarComponent]
        })
    ], SidebarModule);
    return SidebarModule;
}());
exports.SidebarModule = SidebarModule;
