"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var common_1 = require("@angular/common");
var tree_1 = require("primeng/tree");
var get = require("lodash/get");
var template_form_1 = require("./template-form");
var TreeLevel = /** @class */ (function () {
    function TreeLevel() {
        this.leaf = false;
    }
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], TreeLevel.prototype, "textProperty", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], TreeLevel.prototype, "leaf", void 0);
    __decorate([
        core_1.ContentChild(core_1.TemplateRef),
        __metadata("design:type", core_1.TemplateRef)
    ], TreeLevel.prototype, "template", void 0);
    TreeLevel = __decorate([
        core_1.Component({
            selector: 'rz-tree-level',
            template: ''
        })
    ], TreeLevel);
    return TreeLevel;
}());
exports.TreeLevel = TreeLevel;
var TreeComponent = /** @class */ (function () {
    function TreeComponent() {
        this.style = {};
        this.data = [];
        this.nodeExpand = new core_1.EventEmitter();
        this.nodeSelect = new core_1.EventEmitter();
        this.nodeLoaded = new core_1.EventEmitter();
        this.results = [];
    }
    TreeComponent.prototype.ngAfterContentInit = function () {
        this.update();
    };
    TreeComponent.prototype.ngOnChanges = function (changes) {
        this.update();
    };
    TreeComponent.prototype.update = function () {
        var _this = this;
        if (this.data && this.levels && this.levels.length > 0) {
            var root_1 = this.levels.first;
            this.results = this.data.map(function (item) {
                var node = {
                    label: get(item, root_1.textProperty),
                    data: item,
                    type: "level-0",
                    children: [],
                    leaf: root_1.leaf
                };
                _this.nodeLoaded.next(new TreeNodeLoadedData(node, 0));
                return node;
            });
        }
    };
    TreeComponent.prototype.onNodeSelect = function (event) {
        if (event.node) {
            this.nodeSelect.next({
                data: event.node.data,
                level: geLevelIndex(event.node)
            });
        }
    };
    TreeComponent.prototype.onNodeExpand = function (event) {
        if (event.node) {
            var levelIndex = geLevelIndex(event.node);
            if (this.levels) {
                var level = levelIndex + 1 < this.levels.length
                    ? this.levels.toArray()[levelIndex + 1]
                    : this.levels.last;
                if (level) {
                    var textProperty = level.textProperty, leaf = level.leaf;
                    this.nodeExpand.next(new TreeLoadChildrenData(levelIndex, event.node, textProperty, leaf, this.nodeLoaded));
                }
            }
        }
    };
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], TreeComponent.prototype, "style", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Array)
    ], TreeComponent.prototype, "data", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", Object)
    ], TreeComponent.prototype, "nodeExpand", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", Object)
    ], TreeComponent.prototype, "nodeSelect", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", Object)
    ], TreeComponent.prototype, "nodeLoaded", void 0);
    __decorate([
        core_1.ContentChildren(TreeLevel),
        __metadata("design:type", core_1.QueryList)
    ], TreeComponent.prototype, "levels", void 0);
    TreeComponent = __decorate([
        core_1.Component({
            selector: 'rz-tree',
            providers: [template_form_1.hideTemplateFromTemplateForm()],
            template: "\n    <p-tree\n      [value]=\"results\"\n      selectionMode=\"single\"\n      [(selection)]=\"selectedNode\"\n      (onNodeSelect)=\"onNodeSelect($event)\"\n      (onNodeExpand)=\"onNodeExpand($event)\"\n      [style]=\"style\"\n    >\n      <ng-template\n        *ngFor=\"let level of levels; let index = index\"\n        let-node\n        [pTemplate]=\"level.template ? 'level-' + index : 'none'\"\n      >\n        <ng-template\n          [ngTemplateOutlet]=\"level.template\"\n          [ngTemplateOutletContext]=\"{$implicit: node.data}\"\n        >\n        </ng-template>\n      </ng-template>\n    </p-tree>\n  "
        })
    ], TreeComponent);
    return TreeComponent;
}());
exports.TreeComponent = TreeComponent;
var TreeNodeLoadedData = /** @class */ (function () {
    function TreeNodeLoadedData(node, level) {
        this.node = node;
        this.data = node.data;
        this.level = level;
    }
    Object.defineProperty(TreeNodeLoadedData.prototype, "leaf", {
        get: function () {
            return this.node.leaf;
        },
        set: function (value) {
            this.node.leaf = value;
        },
        enumerable: true,
        configurable: true
    });
    return TreeNodeLoadedData;
}());
var TreeLoadChildrenData = /** @class */ (function () {
    function TreeLoadChildrenData(level, node, textProperty, leaf, nodeLoaded) {
        this.level = level;
        this.data = node.data;
        this.textProperty = textProperty;
        this.node = node;
        this.leaf = leaf;
        this.nodeLoaded = nodeLoaded;
    }
    Object.defineProperty(TreeLoadChildrenData.prototype, "children", {
        get: function () {
            return this.node.children;
        },
        set: function (value) {
            var _this = this;
            if (value && value != this.node.children) {
                this.node.children = (Array.isArray(value) ? value : [value]).map(function (item) {
                    var node = {
                        label: get(item, _this.textProperty),
                        type: "level-" + (_this.level + 1),
                        data: item,
                        children: [],
                        leaf: _this.leaf
                    };
                    _this.nodeLoaded.next(new TreeNodeLoadedData(node, _this.level + 1));
                    return node;
                });
            }
        },
        enumerable: true,
        configurable: true
    });
    return TreeLoadChildrenData;
}());
function geLevelIndex(node) {
    var i = 0;
    while (node && node.parent) {
        node = node.parent;
        i++;
    }
    return i;
}
var TreeModule = /** @class */ (function () {
    function TreeModule() {
    }
    TreeModule = __decorate([
        core_1.NgModule({
            imports: [common_1.CommonModule, tree_1.TreeModule],
            declarations: [TreeComponent, TreeLevel],
            exports: [TreeComponent, TreeLevel]
        })
    ], TreeModule);
    return TreeModule;
}());
exports.TreeModule = TreeModule;
