"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var inflection_1 = require("inflection");
var lodash_1 = require("lodash");
var env = require("./env");
var toolbox_1 = require("./toolbox");
function escapeBoundExpression(value) {
    if (value === void 0) { value = ''; }
    return value.replace(/\$/g, '&#36;').replace(/({|})/g, "{{ '$1' }}");
}
function visible(page) {
    var access = page.access;
    if (!access) {
        access = ['Everybody'];
    }
    if (typeof access == 'string') {
        access = [access];
    }
    var roles = access.map(function (role) { return "\"" + role + "\""; }).join(', ');
    return "@Security.IsInRole(new string[]{" + roles + "})";
}
function prepareBlazorComponent(dataSources, application, pageProperties, component, options, index, interpolate) {
    if (index === void 0) { index = function () { return 0; }; }
    if (interpolate === void 0) { interpolate = true; }
    var properties = [];
    var schema = toolbox_1.componentSchemas[component.type];
    var components = [];
    var template = [];
    var editTemplate = [];
    var filterTemplate = [];
    var headerTemplate = [];
    var footerTemplate = [];
    var encode = interpolate
        ? function (value) { return env.interpolateAsBlazorAttribute(pageProperties, value); }
        : escapeBoundExpression;
    var isNavigation = component.type == 'panelmenu' || component.type == 'menu';
    if (application != null && isNavigation) {
        if (!Array.isArray(component.items) || component.items.length < 1) {
            var items = application.pages
                .filter(function (page) { return application.navigation.indexOf(page.name) >= 0; })
                .map(function (page) {
                if (!page.access || !application.security) {
                    return {
                        text: page.name,
                        path: env.file(page.name)
                    };
                }
                else {
                    return {
                        text: page.name,
                        path: env.file(page.name),
                        visible: visible(page)
                    };
                }
            });
            component = __assign({}, component, { items: items });
        }
    }
    if (application != null &&
        application.security &&
        component.type == 'profilemenu') {
        if (!Array.isArray(component.items) || component.items.length < 1) {
            var items = application.security.provider == 'default'
                ? [
                    {
                        path: 'profile',
                        icon: 'settings',
                        text: 'Profile'
                    },
                    {
                        path: 'application-users',
                        icon: 'person',
                        text: 'Users'
                    },
                    {
                        path: 'application-roles',
                        icon: 'security',
                        text: 'Roles'
                    },
                    {
                        text: 'Logout',
                        value: 'Logout',
                        icon: 'power_settings_new'
                    }
                ]
                : [
                    {
                        text: 'Logout',
                        value: 'Logout',
                        icon: 'power_settings_new'
                    }
                ];
            component = __assign({}, component, { items: items });
        }
        if (!Array.isArray(component.events) || component.events.length < 1) {
            var events_1 = {
                click: [
                    {
                        type: 'execute',
                        code: 'Security.Logout();',
                        condition: "${args.Value} == \"Logout\""
                    }
                ]
            };
            component = __assign({}, component, { events: events_1 });
        }
    }
    if (component.type == 'languagepicker' &&
        application != null &&
        application.localization) {
        component = __assign({}, component, { items: application.localization.cultures.map(function (culture) {
                return __assign({}, culture);
            }) });
    }
    if (component.type == 'image' && options && options.baseHref) {
        if (component.path && component.path.startsWith('/assets')) {
            var baseHref = options.baseHref;
            if (baseHref.endsWith('/')) {
                baseHref = baseHref.slice(0, -1);
            }
            var path = "" + baseHref + component.path;
            component = __assign({}, component, { path: path });
        }
    }
    if (isNavigation && options && options.baseHref) {
        var baseHref_1 = options.baseHref;
        if (baseHref_1.endsWith('/')) {
            baseHref_1 = baseHref_1.slice(0, -1);
        }
        var traverse_1 = function (component, callback) {
            var items = component.items;
            if (Array.isArray(items)) {
                items.forEach(function (item) {
                    callback(item);
                    traverse_1(item, callback);
                });
            }
        };
        traverse_1(component, function (item) {
            if (item.image) {
                item.image = "" + baseHref_1 + item.image;
            }
        });
    }
    Object.keys(component)
        .filter(function (key) {
        return !/^(section|root|fromLayout|content|type|row|col|level|events|height|components|template|headerTemplate|editTemplate|footerTemplate|filterTemplate|inTemplate|name|schema)$/.test(key);
    })
        .forEach(function (name) {
        var value = component[name];
        var propertySchema = schema && schema.properties[name];
        var serializeInline = lodash_1.isObject(value) && !Array.isArray(value);
        if (propertySchema) {
            serializeInline =
                serializeInline || propertySchema.serialize == 'inline';
        }
        if (serializeInline && value) {
            value = env.stringify(value);
            var bound = env.isBoundExpression(value);
            value = bound
                ? env.interpolateAsBlazorAttribute(pageProperties, value)
                : value;
            if (bound) {
                value = value.replace(/('{{)|(}}')/g, '');
            }
            properties.push({
                name: name,
                bound: true,
                value: value
            });
        }
        else if (Array.isArray(value) && lodash_1.isObject(value[0])) {
            value.forEach(function (item) {
                var type = item.type;
                var child;
                if ((isNavigation || component.type == 'profilemenu') &&
                    name == 'items' &&
                    application &&
                    application.security) {
                    var page = application.pages.find(function (page) { return env.file(page.name) == item.path; });
                    if (page && page.access) {
                        item = __assign({}, item, { visible: item.visible ? item.visible : "" + visible(page) });
                    }
                }
                child = prepareBlazorComponent(dataSources, application, pageProperties, Object.assign({}, item, {
                    inTemplate: component.inTemplate,
                    type: component.type == 'form'
                        ? type + "-" + component.type + "-" + inflection_1.singularize(name)
                        : component.type.endsWith('item')
                            ? component.type
                            : component.type + "-" + inflection_1.singularize(name)
                }), options);
                if (type !== undefined) {
                    child.properties.push({ name: 'type', value: type });
                }
                if (item.template) {
                    child.properties = child.properties.filter(function (property) { return property.name != 'template'; });
                }
                child.parent = component;
                child.name = null;
                components.push(child);
            });
        }
        else {
            var bound = env.isBoundExpression(value);
            value = bound
                ? env.interpolateAsBlazorAttribute(pageProperties, value)
                : value;
            if (bound &&
                value.indexOf('{{') == 0 &&
                value.lastIndexOf('{{') == 0 &&
                value.endsWith('}}')) {
                value = value.slice(2, -2);
            }
            properties.push({
                name: name,
                bound: bound,
                value: value
            });
        }
    });
    if (component.template && typeof component.template == 'string') {
        component.content = component.template
            ? "<Template Context=\"data\">\n" + env.interpolateAsBlazorAttribute(pageProperties, component.template) + "\n</Template>"
            : '';
    }
    if (component.editTemplate && typeof component.editTemplate == 'string') {
        component.content = component.editTemplate
            ? "<EditTemplate Context=\"data\">\n" + env.interpolateAsBlazorAttribute(pageProperties, component.editTemplate) + "\n</EditTemplate>"
            : '';
    }
    if (component.filterTemplate && typeof component.filterTemplate == 'string') {
        component.content = component.filterTemplate
            ? "<FilterTemplate>\n" + env.interpolateAsBlazorAttribute(pageProperties, component.filterTemplate) + "\n</FilterTemplate>"
            : '';
    }
    if (component.headerTemplate && typeof component.headerTemplate == 'string') {
        var headerTemplate_1 = component.headerTemplate
            ? "<HeaderTemplate>\n" + env.interpolateAsBlazorAttribute(pageProperties, component.headerTemplate) + "\n</HeaderTemplate>"
            : '';
        if (component.content) {
            component.content = "" + component.content + headerTemplate_1;
        }
        else {
            component.content = headerTemplate_1;
        }
    }
    if (component.footerTemplate && typeof component.footerTemplate == 'string') {
        var footerTemplate_1 = '';
        if (component.type == 'grid-column') {
            footerTemplate_1 = component.footerTemplate
                ? "\n<FooterTemplate>\n" + env.interpolateAsBlazorAttribute(pageProperties, component.footerTemplate) + "\n</FooterTemplate>"
                : '';
        }
        else {
            footerTemplate_1 = component.footerTemplate
                ? "<FooterTemplate>\n" + env.interpolateAsBlazorAttribute(pageProperties, component.footerTemplate) + "\n</FooterTemplate>"
                : '';
        }
        if (component.content) {
            component.content = "" + component.content + footerTemplate_1;
        }
        else {
            component.content = footerTemplate_1;
        }
    }
    if (Array.isArray(component.components)) {
        components = component.components.map(function (child) {
            return prepareBlazorComponent(dataSources, application, pageProperties, __assign({}, child, { inTemplate: component.inTemplate }), options, index, interpolate);
        });
    }
    if (Array.isArray(component.template) && component.template.length) {
        template = component.template.map(function (child) {
            return prepareBlazorComponent(dataSources, application, pageProperties, __assign({}, child, { inTemplate: true }), options, index, interpolate);
        });
    }
    if (Array.isArray(component.editTemplate) && component.editTemplate.length) {
        editTemplate = component.editTemplate.map(function (child) {
            return prepareBlazorComponent(dataSources, application, pageProperties, __assign({}, child, { inTemplate: true }), options, index, interpolate);
        });
    }
    if (Array.isArray(component.filterTemplate) && component.filterTemplate.length) {
        filterTemplate = component.filterTemplate.map(function (child) {
            return prepareBlazorComponent(dataSources, application, pageProperties, __assign({}, child, { inTemplate: false }), options, index, interpolate);
        });
    }
    if (Array.isArray(component.headerTemplate) &&
        component.headerTemplate.length) {
        headerTemplate = component.headerTemplate.map(function (child) {
            return prepareBlazorComponent(dataSources, application, pageProperties, __assign({}, child, { inTemplate: false }), options, index, interpolate);
        });
    }
    if (Array.isArray(component.footerTemplate) &&
        component.footerTemplate.length) {
        footerTemplate = component.footerTemplate.map(function (child) {
            return prepareBlazorComponent(dataSources, application, pageProperties, __assign({}, child, { inTemplate: false }), options, index, interpolate);
        });
    }
    if (component.type == 'row') {
        var value = component.class;
        if (!value) {
            value = 'row';
        }
        else if (value.indexOf('row') < 0) {
            value = "row " + value;
        }
        var classProperty = properties.find(function (property) { return property.name == 'class'; });
        if (classProperty) {
            classProperty.value = value;
        }
        else {
            properties.push({
                name: 'class',
                bound: false,
                value: value
            });
        }
    }
    var indexByType = index(component);
    var name = component.name || "" + component.type + indexByType;
    var events = Object.keys(component.events || {})
        .map(function (event) {
        var value = "" + name + env.type(event);
        return {
            name: event,
            value: value,
            actions: component.events[event]
        };
    })
        .filter(function (event) {
        var _a = event.actions, actions = _a === void 0 ? [] : _a;
        return actions.length > 0;
    });
    var content = component.content, inTemplate = component.inTemplate;
    return {
        type: component.type,
        root: component.root,
        name: name,
        content: encode(content),
        template: template,
        editTemplate: editTemplate,
        headerTemplate: headerTemplate,
        filterTemplate: filterTemplate,
        footerTemplate: footerTemplate,
        inTemplate: inTemplate,
        events: events,
        properties: properties,
        components: components,
        itemType: component.itemType
    };
}
exports.prepareBlazorComponent = prepareBlazorComponent;
//# sourceMappingURL=Component.js.map